# -*- coding: utf-8 -*-


from dp_tornado.engine.helper import Helper as dpHelper

try:
    import cPickle as cPickle
except ImportError:
    import cpickle as cPickle


class _DynamicObject(object):
    def __init__(self):
        self.__dict__['_type'] = None

        self.__dict__['_object'] = {}
        self.__dict__['_items'] = []

    def append(self, value):
        if self.__dict__['_type'] not in (list, None):
            raise ValueError

        self.__dict__['_type'] = list
        self.__dict__['_items'].append(value)

    def __setattr__(self, key, value):
        self.__dict__['_object'][key] = value

    def __getattr__(self, item):
        try:
            result = self.__getattribute__(item)

            return result
        except AttributeError:
            pass

        if item in self.__dict__['_object']:
            return self.__dict__['_object'][item]

        self.__dict__['_object'][item] = _DynamicObject()
        return self.__dict__['_object'][item]

    def __str__(self):
        return str('_DynamicObject:%s' % self.dumps())

    def __repr__(self):
        return str('_DynamicObject:%s' % self.dumps())

    def dumps(self):
        if self.__dict__['_type'] == list:
            output = []

            for item in self.__dict__['_items']:
                if isinstance(item, _DynamicObject):
                    output.append(item.dumps())

                else:
                    output.append(item)

            return output

        else:
            output = {}

            for item in self.__dict__['_object'].keys():
                if isinstance(self.__dict__['_object'][item], _DynamicObject):
                    output[item] = self.__dict__['_object'][item].dumps()

                else:
                    output[item] = self.__dict__['_object'][item]

            return output


class ObjectHelper(dpHelper):
    def create(self):
        return _DynamicObject()

    def pickle(self, o):
        try:
            return cPickle.dumps(o)
        except:
            return False

    def unpickle(self, o):
        try:
            return cPickle.loads(o)
        except:
            return False


# -*- coding: utf-8 -*-


try:
    import configparser
except ImportError:
    import ConfigParser as configparser

import os
import sys
import logging
import tornado.options

from dp_tornado.engine.static_handler import StaticHandler


class Bootstrap(object):
    @staticmethod
    def init_template(engine_path, application_path):
        valid = True

        for e in ('config', 'controller', 'model', 'helper'):
            if os.path.isdir(os.path.join(application_path, e)):
                valid = False
                break

        if not valid:
            logging.warning('Default directory structure initialzation skipped. Directory not empty.')

        else:
            import shutil

            template_path = os.path.join(engine_path, 'engine', 'template')

            for root, dirs, files in os.walk(template_path):
                path = root[len(template_path)+1:]
                app_path = os.path.join(application_path, path)

                if path and os.path.isdir(app_path):
                    continue

                if not os.path.isdir(app_path):
                    os.mkdir(app_path)

                for file in files:
                    src = os.path.join(root, file)
                    dest = os.path.join(app_path, file)

                    if not os.path.isfile(dest):
                        shutil.copy(src, dest)

    @staticmethod
    def init_args():
        import argparse

        parser = argparse.ArgumentParser()

        parser.add_argument('--app-path', help='App Path')

        parser.add_argument('--scheduler-path', help='Scheduler Path')
        parser.add_argument('--scheduler-timeout', type=int, help='Scheduler Timeout')

        parser.add_argument('-i', '--identifier', help='Identifier')
        parser.add_argument('-p', '--port', type=int, help='Binding port')

        return parser.parse_args()

    @staticmethod
    def init_ini(application_path, ini_file):
        args = Bootstrap.init_args()

        # INI
        config = configparser.RawConfigParser()
        config.read(os.path.join(application_path, ini_file or 'config.ini'))

        def get_cfg(c, option, section='server', default=None):
            try:
                get = c.get(section, option)

                if default is True or default is False:
                    return True if get == '1' else False

                elif isinstance(default, str):
                    return str(get)

                elif isinstance(default, int):
                    return int(get)

                else:
                    return get
            except (configparser.NoSectionError, configparser.NoOptionError):
                return default

        combined_path = get_cfg(config, 'path', section='static', default='combined')
        static_prefix = get_cfg(config, 'prefix', section='static', default='/s/')
        static_minify = get_cfg(config, 'minify', section='static', default=True)

        if combined_path.find('{server_name}') != -1:
            import socket
            combined_path = combined_path.replace('{server_name}', socket.gethostname())

        if static_prefix[-1] != '/':
            static_prefix = '%s/' % static_prefix

        if combined_path[0] == '/':
            combined_path = combined_path[1:]

        if combined_path[-1] == '/':
            combined_path = combined_path[:-1]

        combined_prefix = '%s%s/' % (static_prefix, combined_path)
        combined_url = combined_path
        combined_path = os.path.join(application_path, 'static', combined_path)

        # Setup Options
        tornado.options.define('max_worker', default=get_cfg(config, 'max_worker', default=1))
        tornado.options.define('num_processes', default=get_cfg(config, 'num_processes', default=0))
        tornado.options.define('port', default=args.port or get_cfg(config, 'port', default=8080))
        tornado.options.define('debug', default=get_cfg(config, 'debug', default=False))
        tornado.options.define('gzip', default=get_cfg(config, 'gzip', default=True))
        tornado.options.define('crypto_key', default=get_cfg(config, 'key', section='crypto', default='CR$t0-$CR@T'))
        tornado.options.define('session_dsn', default=get_cfg(config, 'dsn', section='session', default=None))
        tornado.options.define('session_exp_in', default=get_cfg(config, 'expire_in', section='session', default=7200))
        tornado.options.define('session_ip_restrict', default=get_cfg(config, 'ip_restrited', section='session', default=1))
        tornado.options.define('max_body_size', default=get_cfg(config, 'max_body_size', default=1024*1024*10))
        tornado.options.define('application_path', application_path)
        tornado.options.define('python', sys.executable)

        m17n = get_cfg(config, 'm17n', default='').strip()
        m17n = m17n.split(',') if m17n else ['dummy']
        m17n = [e.strip() for e in m17n]

        tornado.options.define('m17n', m17n)

        # Static AWS
        tornado.options.define('static_aws_id', default=get_cfg(config, 'aws_id', section='static'))
        tornado.options.define('static_aws_secret', default=get_cfg(config, 'aws_secret', section='static'))
        tornado.options.define('static_aws_bucket', default=get_cfg(config, 'aws_bucket', section='static'))
        tornado.options.define('static_aws_region', default=get_cfg(config, 'aws_region', section='static'))
        tornado.options.define('static_aws_endpoint', default=get_cfg(config, 'aws_endpoint', section='static'))

        # Scheduler
        tornado.options.define('scheduler_timezone', default=get_cfg(config, 'timezone', section='scheduler', default=''))
        tornado.options.define('scheduler_mode', default=get_cfg(config, 'mode', section='scheduler', default='web'))

        exception_delegate = get_cfg(config, 'exception_delegate', default='', section='logging') or None
        access_logging = get_cfg(config, 'access', default=1, section='logging')
        sql_logging = get_cfg(config, 'sql', default=0, section='logging')

        if exception_delegate:
            from dp_tornado.engine.engine import Engine as dpEngine
            exception_delegate = eval('dpEngine().%s' % exception_delegate)

        tornado.options.define('exception_delegate', exception_delegate)

        # Initialize Logging
        logging.basicConfig(
            level=logging.DEBUG if access_logging else logging.WARN, format='[%(asctime)s][%(levelname)s] %(message)s')

        # SQLAlchemy logging level
        if sql_logging:
            logging.getLogger('sqlalchemy.engine').setLevel(logging.INFO)

        return {
            'template_path': os.path.join(application_path, 'view'),
            'static_path': os.path.join(application_path, 'static'),
            'static_handler_class': StaticHandler,
            'static_url_prefix': static_prefix,
            'static_minify': static_minify,
            'static_combined_url': combined_url,
            'combined_static_path': combined_path,
            'combined_static_url_prefix': combined_prefix,
            'compressors': {
                'minifier': None
            },
            'debug': tornado.options.options.debug,
            'gzip': tornado.options.options.gzip,
            'cookie_secret': get_cfg(config, 'cookie_secret', default='default_cookie_secret'),
            'ui_modules': {}
        }



# -*- coding: utf-8 -*-


import os

from urllib2 import urlopen
from PIL import Image
from PIL import ImageDraw
from PIL import ImageOps

from dp_tornado.engine.helper import Helper as dpHelper


class ResizeHelper(dpHelper):
    def rounded_mask(self, size, radius, factor=2):
        width, height = size
        radius = min(radius, *size)

        mask = Image.new("L", (width * factor, height * factor))
        mask_draw = ImageDraw.Draw(mask)
        mask_draw.ellipse((0, 0, radius * factor, radius * factor), fill='#FFFFFF')

        flip = mask.transpose(Image.FLIP_LEFT_RIGHT)
        mask.paste(flip, (0, 0), flip)
        mask_draw.rectangle((radius * factor / 2, 0, (width - (radius / 2)) * factor, radius * factor), fill='#FFFFFF')
        flip = mask.transpose(Image.FLIP_TOP_BOTTOM)
        mask.paste(flip, (0, 0), flip)
        mask_draw.rectangle((0, radius * factor / 2, width * factor, (height - (radius / 2)) * factor), fill='#FFFFFF')
        mask = mask.resize((width, height), Image.ANTIALIAS)

        return mask

    def resizing(self, filename, size, **kwargs):
        mode = kwargs['mode'] if 'mode' in kwargs else None
        scale = int(kwargs['scale']) if 'scale' in kwargs else 1
        edge_crop = kwargs['edge_crop'] if 'edge_crop' in kwargs else None
        border = int(kwargs['border']) if 'border' in kwargs else 0
        border_color = kwargs['border_color'] if 'border_color' in kwargs else '#000000'
        limit = True if 'limit' in kwargs and kwargs['limit'] else False
        background = kwargs['background'] if 'background' in kwargs else None
        colorize = kwargs['colorize'] if 'colorize' in kwargs else None
        radius = int(kwargs['radius'] or 0) if 'radius' in kwargs else None
        save = kwargs['save'] if 'save' in kwargs else None
        fmt = kwargs['fmt'] if 'fmt' in kwargs else None

        if not os.path.isfile(filename):
            f = urlopen(filename)
            img = Image.open(f)
        else:
            img = Image.open(filename)

        if not img:
            raise Exception('The specified image is invalid.')

        # Edge Cropping
        if edge_crop:
            e_top = 0
            e_left = 0
            e_right = 0
            e_bottom = 0

            if self.helper.system.py_version <= 2:
                types_str = (basestring, )
                types_num = (int, long)
            else:
                types_str = (str, )
                types_num = (int, )

            if isinstance(edge_crop, types_str):
                edge_crop = edge_crop.split(',')

                for i in range(len(edge_crop)):
                    edge_crop[i] = int(edge_crop[i])

            if isinstance(edge_crop, types_num):
                e_top = e_left = e_right = e_bottom = edge_crop

            elif isinstance(edge_crop, (tuple, list)):
                if len(edge_crop) == 1:
                    e_top = e_left = e_right = e_bottom = edge_crop[0]

                elif len(edge_crop) == 2:
                    e_top = e_bottom = edge_crop[0]
                    e_left = e_right = edge_crop[1]

                elif len(edge_crop) == 4:
                    e_top = edge_crop[0]
                    e_right = edge_crop[1]
                    e_bottom = edge_crop[2]
                    e_left = edge_crop[3]

            img = img.crop((e_left, e_top, img.size[0] - e_right, img.size[1] - e_bottom))

        ext = (fmt or os.path.splitext(filename)[1][1:]).lower()
        width_new, height_new = size
        width_origin, height_origin = img.size

        if scale > 1:
            if limit:
                scale_max_width = float(width_origin) / float(width_new)
                scale_max_height = float(height_origin) / float(height_new)

                scale_max = min(scale, scale_max_width, scale_max_height)
            else:
                scale_max = scale

            if scale_max > 1:
                width_new = int(width_new * scale_max)
                height_new = int(height_new * scale_max)

        if not width_new:
            width_new = width_origin * height_new / height_origin
            mode = self.helper.image.resize.mode.resize

        if not height_new:
            height_new = height_origin * width_new / width_origin
            mode = self.helper.image.resize.mode.resize

        if border:
            width_new -= border * 2
            height_new -= border * 2

        if not mode:
            mode = self.helper.image.resize.mode.resize

        if mode not in self.helper.image.resize.mode.modes:
            raise Exception('The specified mode is not supported.')

        # Image Resizing
        if mode == self.helper.image.resize.mode.center:
            width_calc = width_new
            height_calc = height_origin * width_calc / width_origin

            if height_calc > height_new:
                height_calc = height_new
                width_calc = width_origin * height_calc / height_origin

            img = img.resize((width_calc, height_calc), Image.ANTIALIAS)

            if radius:
                fmt = 'PNG'
                img = img.convert('RGBA')
                img.putalpha(self.rounded_mask(img.size, radius))
                radius = None

            img = ImageOps.expand(
                img, border=((width_new - width_calc) / 2, (height_new - height_calc) / 2), fill=background)

        elif mode == self.helper.image.resize.mode.fill:
            ratio_origin = float(width_origin) / float(height_origin)
            ratio_new = float(width_new) / float(height_new)

            if ratio_origin > ratio_new:
                tw = int(round(height_new * ratio_origin))
                img = img.resize((tw, height_new), Image.ANTIALIAS)
                left = int(round((tw - width_new) / 2.0))
                img = img.crop((left, 0, left + width_new, height_new))

            elif ratio_origin < ratio_new:
                th = int(round(width_new / ratio_origin))
                img = img.resize((width_new, th), Image.ANTIALIAS)
                top = int(round((th - height_new) / 2.0))
                img = img.crop((0, top, width_new, top+height_new))

            else:
                img = img.resize((width_new, height_new), Image.ANTIALIAS)

        elif mode == self.helper.image.resize.mode.resize:
            if width_new > width_origin or height_new > height_origin:
                width_new = width_origin
                height_new = height_origin

            img = img.resize((width_new, height_new), Image.ANTIALIAS)

        # Colorizing
        if colorize:
            img = img.convert('RGBA')
            r, g, b, a = img.split()
            gray = ImageOps.grayscale(img)
            result = ImageOps.colorize(gray, colorize, (255, 255, 255, 0))
            result.putalpha(a)
            img = result

        # PNG : 1, L, P, RGB, RGBA
        # GIF : L, P
        # JPEG : L, RGB, CMYK

        # Invalid format specified
        if not fmt and ext in ('jpg', 'jpeg') and img.mode == 'P':  # JPEG not supported P mode
            img = img.convert('RGBA')

        # Border without radius
        if border and not radius:
            if img.mode == 'P':
                img = img.convert('RGBA')

            img = ImageOps.expand(img, border=(border, border), fill=border_color)

        # Add rounded corner
        if radius:
            fmt = 'PNG'
            img = img.convert('RGBA')
            img.putalpha(self.rounded_mask(img.size, min(radius, *img.size)))

            # Border with radius
            if border:
                img_o = img

                bordered_size = (img.size[0] + (border * 2), img.size[1] + (border * 2))
                img = Image.new('RGBA', bordered_size, border_color)
                img.putalpha(self.rounded_mask(bordered_size, min(radius, *bordered_size)))
                img.paste(img_o, (border, border), img_o)

        if save:
            img.save(save, format=fmt, quality=100)

        return img



# -*- coding: utf-8 -*-


import sys
import tornado.options

from .engine import Engine as dpEngine

allowed_m17n = tornado.options.options.m17n
session_default_expire_in = tornado.options.options.session_exp_in or 7200
py_version = sys.version_info[0]

if py_version >= 3:
    long = int


class Controller(dpEngine):
    def __init__(self, application, request, **kwargs):
        super(Controller, self).__init__()

        self.prefix = kwargs.get('prefix')
        self.parent = kwargs.get('parent')

        self._write = []
        self._finish = None
        self._render = None

        self.head_requested = False
        self.post_requested = False
        self.get_requested = False
        self.patch_requested = False
        self.delete_requested = False
        self.put_requested = False

    def initialize(self, prefix, parent=None):
        self.prefix = prefix
        self.parent = parent

    @property
    def request(self):
        return self.parent.request

    def get_argument(self, name, default=None, strip=True, cast=None, fmt=None, delimiter=None):
        ret = self.parent.get_argument(name, default, strip)

        try:
            if ret and cast == int and fmt == 'yyyymmdd':
                try:

                    return self.helper.datetime.time(
                        self.helper.datetime.datetime(
                            yyyymmdd=self.helper.numeric.extract_numbers(ret or '')))

                except:
                    return False

            elif ret and cast == int:
                return int(ret)
            elif ret and cast == long:
                return long(ret)
            elif ret and cast == float:
                return float(ret)
            elif ret and cast == bool:
                return True if ret in ('1', 'yes', 'true', 'True', 'TRUE') else False
            elif delimiter:
                ret = ret.split(delimiter) if ret else ret

        except ValueError:
            return default

        return ret

    def get_user_agent(self, parsed=True):
        return self.parent.get_user_agent(parsed=parsed)

    @property
    def remote_ip(self):
        return self.parent.remote_ip

    def set_secure_cookie(self, name, value, expires_days=30, version=2, **kwargs):
        return self.parent.set_secure_cookie(name, value, expires_days=expires_days, version=version, **kwargs)

    def get_secure_cookie(self, name, value=None, max_age_days=31, min_version=None):
        return self.parent.get_secure_cookie(name, value=value, max_age_days=max_age_days, min_version=min_version)

    def get_sessionid(self):
        return self.parent.get_sessionid()

    def set_sessionid(self, sessionid=None):
        return self.parent.set_sessionid(sessionid=sessionid)

    def get_session_value(self, name, expire_in=None, sessionid=None):
        return self.parent.get_session_value(name, expire_in=expire_in, sessionid=sessionid)

    def get_session_value_ttl(self, name):
        return self.parent.get_session_value_ttl(name)

    def set_session_value(self, name, value, expire_in=session_default_expire_in):
        return self.parent.set_session_value(name, value, expire_in=expire_in)

    def session(self, name, value=None, expire_in=session_default_expire_in):
        return self.parent.session(name, value=value, expire_in=expire_in)

    def _prefix(self, url):
        if 'X-Proxy-Prefix' in self.request.headers:
            prefix = self.request.headers['X-Proxy-Prefix']
            prefix = prefix[:-1] if prefix.endswith('/') else prefix

            if url.startswith(prefix):
                url = url[len(prefix):] or '/'

        return url

    def redirect(self, url, prefix=False, permanent=False, status=None, safe=False):
        if self.parent._headers_written:
            return

        if prefix:
            url = self._prefix(url)

        if safe and url and not url.startswith('/'):
            url = '/'

        try:
            self.parent.redirect(url, permanent, status)
        except (AttributeError, TypeError):
            pass

    def request_uri(self, s=False, d=' ', p='_', e=False, q=True):
        r = ('%s%s' % (d, p)).join(self.request.uri.split('/')).strip() if s else self.request.uri
        r = self._prefix(r)
        r = r if q else r.split('?')[0:1][0]

        return self.helper.url.quote(r) if e else r

    def render(self, template_name, kwargs=None):
        self._render = {'t': template_name, 'k': kwargs}

    def render_string(self, template_name, kwargs=None):
        if kwargs:
            return self.view.render_string(self, template_name, kwargs)
        else:
            return self.view.render_string(self, template_name)

    def write(self, chunk):
        self._write.append(chunk)

    def finish(self, chunk=None):
        self._finish = chunk

    def m17n_lang(self, lang=None):
        if lang is None:
            m17n = self.parent.get_cookie('__m17n__')
            return m17n if m17n in allowed_m17n else allowed_m17n[0]

        return self.parent.m17n_lang(lang)



# -*- coding: utf-8 -*-


from __future__ import absolute_import
from dp_tornado.engine.helper import Helper as dpHelper

import json


class JsonHelper(dpHelper):
    def serialize(self, a, raise_exception=False, separators=(',', ':')):
        try:
            return self.dumps(a, separators=separators)
        except Exception as e:
            if raise_exception:
                raise e

            return False

    def deserialize(self, a, raise_exception=False):
        try:
            return self.loads(a)
        except Exception as e:
            if raise_exception:
                raise e

            return False

    def dumps(self, a, separators=None, indent=None):
        return json.dumps(a, separators=separators, indent=indent)

    def loads(self, a, encoding='utf8'):
        return json.loads(a, encoding=encoding)



# -*- coding: utf-8 -*-


import logging
import traceback
import threading
import tornado.options

from .singleton import Singleton as dpSingleton

try:
    import Queue as queue
except ImportError:
    import queue


class Logger(dpSingleton):
    def __init__(self, engine=None):
        self.engine = engine
        self.delegate_handler = tornado.options.options.exception_delegate

        if self.delegate_handler:
            self.delegate_queue = queue.Queue()
            self.delegate = LoggerDelegate(self)
            self.delegate.start()

    def strip(self, msg, strip=False):
        return msg.strip() if strip else msg

    def exception(self, exception, *args, **kwargs):
        msg = self.strip(str(exception), True)
        tb = traceback.format_exc()

        if self.delegate_handler:
            self.delegate_queue.put((logging.ERROR, msg, tb))

        logging.exception(msg, *args, **kwargs)
        logging.exception(tb)

    def error(self, msg, *args, **kwargs):
        logging.error(self.strip(str(msg), True), *args, **kwargs)

    def info(self, msg, *args, **kwargs):
        logging.info(self.strip(msg), *args, **kwargs)

    def warning(self, msg, *args, **kwargs):
        logging.warning(self.strip(msg), *args, **kwargs)

    def debug(self, msg, *args, **kwargs):
        logging.debug(self.strip(msg), *args, **kwargs)

    def delegate_interrupt(self):
        if self.delegate_handler:
            self.delegate_queue.put(False)


class LoggerDelegate(threading.Thread):
    def __init__(self, logger):
        threading.Thread.__init__(self)
        self.logger = logger

    def run(self):
        while True:
            payload = self.logger.delegate_queue.get()

            if not payload:
                break

            self.logger.delegate_handler(*payload)



# -*- coding: utf-8 -*-


class SchemaDriver(object):
    @staticmethod
    def migrate(dsn, table, fields, indexes, foreign_keys):
        raise NotImplementedError

    @staticmethod
    def migrate_data(dsn, table):
        raise NotImplementedError

    @staticmethod
    def migrate_fields(proxy, table, fields, exist):
        raise NotImplementedError

    @staticmethod
    def _migrate_fields_create(proxy, table, fields, exist):
        raise NotImplementedError

    @staticmethod
    def migrate_indexes(proxy, table, indexes, exist):
        raise NotImplementedError

    @staticmethod
    def migrate_foreign_keys(proxy, table, foreign_keys, exist):
        raise NotImplementedError



# -*- coding: utf-8 -*-


from dp_tornado.engine.helper import Helper as dpHelper

import re


# Regular Expression for URL validation
# Author: Diego Perini
# License: MIT
# Copyright (c) 2010-2013 Diego Perini (http://www.iport.it)

# https://gist.github.com/dperini/729294

regex = re.compile(
    u"^"
    # protocol identifier
    u"(?:(?:https?|ftp)://)"
    # user:pass authentication
    u"(?:\S+(?::\S*)?@)?"
    u"(?:"
    # IP address exclusion
    # private & local networks
    u"(?!(?:10|127)(?:\.\d{1,3}){3})"
    u"(?!(?:169\.254|192\.168)(?:\.\d{1,3}){2})"
    u"(?!172\.(?:1[6-9]|2\d|3[0-1])(?:\.\d{1,3}){2})"
    # IP address dotted notation octets
    # excludes loopback network 0.0.0.0
    # excludes reserved space >= 224.0.0.0
    # excludes network & broadcast addresses
    # (first & last IP address of each class)
    u"(?:[1-9]\d?|1\d\d|2[01]\d|22[0-3])"
    u"(?:\.(?:1?\d{1,2}|2[0-4]\d|25[0-5])){2}"
    u"(?:\.(?:[1-9]\d?|1\d\d|2[0-4]\d|25[0-4]))"
    u"|"
    # host name
    u"(?:(?:[a-z\u00a1-\uffff0-9]-?)*[a-z\u00a1-\uffff0-9]+)"
    # domain name
    u"(?:\.(?:[a-z\u00a1-\uffff0-9]-?)*[a-z\u00a1-\uffff0-9]+)*"
    # TLD identifier
    u"(?:\.(?:[a-z\u00a1-\uffff]{2,}))"
    u")"
    # port number
    u"(?::\d{2,5})?"
    # resource path
    u"(?:/\S*)?"
    u"$"
    , re.UNICODE)


class UrlHelper(dpHelper):
    def validate(self, e):
        return True if re.match(regex, e) else False



# -*- coding: utf-8 -*-


from dp_tornado.engine.helper import Helper as dpHelper
from dp_tornado.engine.engine import Engine as dpEngine


_s3_connection_ = None


def s3_connection():
    global _s3_connection_

    if _s3_connection_:
        return _s3_connection_

    try:
        from boto.s3.connection import S3Connection

        import logging
        logging.getLogger('boto').setLevel(logging.CRITICAL)

        _s3_connection_ = S3Connection

    except ImportError as e:
        raise e

    return _s3_connection_


class S3Bridge(dpEngine):
    def __init__(self, public, secret):
        try:
            from boto.s3.connection import S3Connection

            import logging
            logging.getLogger('boto').setLevel(logging.CRITICAL)

        except ImportError as e:
            raise e

        self.conn = S3Connection(public, secret)

    def bucket(self, bucket_name):
        return self.conn.get_bucket(bucket_name)

    def set_contents_from_file(self, bucket_name, key, fp, url=False):
        try:
            from boto.s3.key import Key
        except ImportError as e:
            raise e

        fp.seek(0)
        bucket = self.bucket(bucket_name)
        obj = Key(bucket)
        obj.key = key
        res = obj.set_contents_from_file(fp)

        if url:
            return obj.generate_url(0)
        else:
            return res

    def copy(self, from_bucket_name, from_key_name, to_bucket_name, to_key_name):
        try:
            from boto.s3.key import Key
        except ImportError as e:
            raise e

        from_bucket = self.bucket(from_bucket_name)
        from_key = Key(from_bucket)
        from_key.key = from_key_name

        to_bucket = self.bucket(to_bucket_name)
        return from_key.copy(to_bucket, to_key_name)


class S3Helper(dpHelper):
    def connect(self, public, secret):
        return S3Bridge(public, secret)

    def set_contents_from_file(self,
                               aws_access_key_id,
                               aws_secret_access_key,
                               bucket_name,
                               region_name,
                               key,
                               fp,
                               url=False,
                               ExtraArgs=None):
        import boto3

        s3 = boto3.client(
            service_name='s3',
            region_name=region_name,
            aws_access_key_id=aws_access_key_id,
            aws_secret_access_key=aws_secret_access_key)

        s3.upload_file(fp, bucket_name, key, ExtraArgs=ExtraArgs)

    def copy(self,
             aws_access_key_id,
             aws_secret_access_key,
             region_name,
             src_bucket_name,
             src_key,
             dest_bucket_name,
             dest_key):
        import boto3

        s3 = boto3.client(
            service_name='s3',
            region_name=region_name,
            aws_access_key_id=aws_access_key_id,
            aws_secret_access_key=aws_secret_access_key)

        copy_source = {
            'Bucket': src_bucket_name,
            'Key': src_key
        }

        s3.copy(copy_source, dest_bucket_name, dest_key)

        return True

    def prepare_post(self,
                     aws_access_key_id,
                     aws_secret_access_key,
                     bucket_name,
                     key,
                     region=None,
                     success_action_redirect=None,
                     max_content_length=None,
                     expires_in=6000,
                     acl=None):
        if region:
            return self._prepare_post_boto3(
                aws_access_key_id=aws_access_key_id,
                aws_secret_access_key=aws_secret_access_key,
                bucket_name=bucket_name,
                key=key,
                region=region,
                success_action_redirect=success_action_redirect,
                max_content_length=max_content_length,
                expires_in=expires_in,
                acl=acl)

        s3 = s3_connection()(
            aws_access_key_id=aws_access_key_id,
            aws_secret_access_key=aws_secret_access_key)

        if not success_action_redirect:
            fields = [{"name": "success_action_status", "value": "201"}]
            conditions = ['{"success_action_status": "201"}']
        else:
            fields = [{"name": "success_action_redirect", "value": success_action_redirect}]
            conditions = ['{"success_action_redirect": "%s"}' % success_action_redirect]

        payload = s3.build_post_form_args(
            bucket_name=bucket_name,
            key=key,
            expires_in=expires_in,
            acl=acl,
            max_content_length=max_content_length,
            fields=fields,
            conditions=conditions)

        return payload

    def _prepare_post_boto3(self,
                            aws_access_key_id,
                            aws_secret_access_key,
                            bucket_name,
                            key,
                            region=None,
                            success_action_redirect=None,
                            max_content_length=None,
                            expires_in=6000,
                            acl=None):
        import boto3

        s3 = boto3.client(
            service_name='s3',
            region_name=region,
            aws_access_key_id=aws_access_key_id,
            aws_secret_access_key=aws_secret_access_key)

        fields = {}
        conditions = []

        if success_action_redirect:
            fields['success_action_redirect'] = success_action_redirect
            conditions.append({'success_action_redirect': success_action_redirect})
        else:
            fields['success_action_status'] = '201'
            conditions.append({'success_action_status': '201'})

        if acl:
            conditions.append({'acl': acl})

        if max_content_length:
            conditions.append(["content-length-range", 0, max_content_length])

        payload = s3.generate_presigned_post(
            Bucket=bucket_name,
            Key=key,
            Fields=fields,
            Conditions=conditions,
            ExpiresIn=expires_in)

        return {
            'action': payload['url'],
            'fields': [{'name': k, 'value': v} for k, v in payload['fields'].items()]
        }

    def generate_url_with_filename(self,
                                   aws_access_key_id,
                                   aws_secret_access_key,
                                   bucket_name,
                                   key,
                                   file_name,
                                   disposition='attachment',
                                   expires_in=6000):
        s3 = s3_connection()(
            aws_access_key_id=aws_access_key_id,
            aws_secret_access_key=aws_secret_access_key)

        response_headers = {
            'response-content-disposition':
                '%s; filename="%s"' % (disposition, self.helper.url.urlencode(self.helper.string.to_str(file_name)))
        }

        return s3.generate_url(expires_in, 'GET', bucket_name, key, response_headers=response_headers)



# -*- coding: utf-8 -*-


from dp_tornado.engine.helper import Helper as dpHelper


class ModeHelper(dpHelper):
    @property
    def resize(self):
        return 'resize'

    @property
    def center(self):
        return 'center'

    @property
    def fill(self):
        return 'fill'

    @property
    def auto(self):
        return 'auto'

    @property
    def modes(self):
        return [self.resize, self.center, self.fill, self.auto]



# -*- coding: utf-8 -*-


import os
import requests
import tornado.web
import tornado.options
import subprocess
import tempfile

from ..cache import dpInValueModelConfig
from ..engine import Engine as dpEngine


class Compressor(dpEngine):
    def __init__(self, handler):
        self.handler = handler
        self.compressors = handler.settings.get('compressors', False) if handler else None
        self.provider = {
            'minifier': 'minify'
        }

        if self.compressors and 'minifier' in self.compressors and self.compressors['minifier']:
            self.provider['minifier'] = self.compressors['minifier']

    def _commands(self, path, files, ext, tempname):
        files = [os.path.join(path, fn) for fn in files]

        if ext == 'js':
            return [self.provider['minifier'], '--output', tempname] + files
        elif ext == 'css':
            return [self.provider['minifier'], '--output', tempname] + files

    def _compress(self, path, files, ext, tempname):
        tmpfiles = []
        offset = 0

        for file in files:
            offset += 1

            tmpname = '%s_%s' % (tempname, offset)
            tmp = open(tmpname, 'w')
            tmp.close()

            commands = self._commands(path, [file], ext, tmpname)
            subprocess.check_output(commands)

            tmpfiles.append(tmpname)

        with open(tempname, 'w') as outfile:
            for tmpfile in tmpfiles:
                with open(tmpfile) as infile:
                    for line in infile:
                        outfile.write(line)

                os.remove(tmpfile)

        return True

    def compress(self, path, files, ext, tempname):
        try:
            if ext == 'js':
                commands = self._commands(path, files, ext, tempname)
                subprocess.check_output(commands)
            else:
                if not self._compress(path, files, ext, tempname):
                    return False

            return True

        except subprocess.CalledProcessError as e:
            self.logging.error('--------------------------------')
            self.logging.error('Static file minification error :')
            self.logging.error('Return code : %s' % e.returncode)
            self.logging.error('Command : %s' % e.cmd)
            self.logging.error('Output : %s' % e.output)
            self.logging.error('--------------------------------')
            for ee in files:
                self.logging.error(ee)
            self.logging.error('--------------------------------')

        except Exception as e:
            self.logging.error('--------------------------------')
            self.logging.error('Static file minification error :')
            self.logging.error(e)
            self.logging.error('--------------------------------')
            for ee in files:
                self.logging.error(ee)
            self.logging.error('--------------------------------')

        return False

    def _read(self, location):
        f = open(location, 'r')
        r = f.readlines()
        f.close()

        return ''.join(r)

    @staticmethod
    def clear(combined_dir):
        if not os.path.isdir(combined_dir):
            os.makedirs(combined_dir)

        for f in os.listdir(combined_dir):
            path = os.path.join(combined_dir, f)

            if not os.path.isdir(path):
                os.remove(path)


class StaticURL(tornado.web.UIModule):
    def render(self, *statics, **options):
        if not self.handler.vars.compressor:
            self.handler.vars.compressor = Compressor(self.handler)
            self.handler.vars.static.minify = self.handler.settings.get('static_minify')
            self.handler.vars.static.path = self.handler.settings.get('static_path')
            self.handler.vars.static.prefix = self.handler.settings.get('static_url_prefix')
            self.handler.vars.static.combined_url = self.handler.settings.get('static_combined_url')
            self.handler.vars.static.combined_path = self.handler.settings.get('combined_static_path')
            self.handler.vars.static.combined_prefix = self.handler.settings.get('combined_static_url_prefix')
            self.handler.vars.static.cache_config = dpInValueModelConfig('sqlite', 'static_url', pure=True)

            self.handler.vars.static.aws_id = tornado.options.options.static_aws_id
            self.handler.vars.static.aws_secret = tornado.options.options.static_aws_secret
            self.handler.vars.static.aws_bucket = tornado.options.options.static_aws_bucket
            self.handler.vars.static.aws_region = tornado.options.options.static_aws_region
            self.handler.vars.static.aws_endpoint = tornado.options.options.static_aws_endpoint

            self.handler.vars.static.aws_configured = True if (self.handler.vars.static.aws_id and
                                                               self.handler.vars.static.aws_secret and
                                                               self.handler.vars.static.aws_bucket and
                                                               self.handler.vars.static.aws_endpoint) else False

        # for Debugging Mode, do not minify css or js.
        if self.handler.settings.get('debug'):
            return '\n'.join(
                [self._template('%s?%s' % (t, self.handler.vars.compressor.helper.datetime.mtime())) for t in statics])
        elif not self.handler.vars.static.minify or (options and 'proxy' in options and options['proxy']):
            return '\n'.join([self._template('%s?%s' % (t, self.handler.application.startup_at)) for t in statics])

        cache_key = 'key_%s_%s' % (len(statics), self.handler.helper.crypto.sha224_hash('/'.join(sorted(statics))))
        prepared = (self.handler.vars.static.prepared.__getattr__(cache_key) or
                    self.handler.cache.get(cache_key, dsn_or_conn=self.handler.vars.static.cache_config))

        if prepared:
            return prepared

        if 'separate' not in options or not options['separate']:
            extensions = {}

            for static in statics:
                ext = static.split('?')[0].split('.')[-1].lower()

                if ext not in extensions:
                    extensions[ext] = []

                extensions[ext].append(static)

            if not extensions:
                return ''

            if len(extensions) > 1:
                options['separate'] = False
                return '\n'.join(self.render(*static, **options) for static in list(extensions.values()))
            else:
                statics = list(extensions.values())[0]

        template = self._template(statics[0], replaced=False)
        ext = statics[0].split('.')[-1].lower()

        if not template:
            return ''

        acquire_key = 'job/%s' % cache_key
        acquire_identifier = self.handler.helper.random.uuid()

        if not self.handler.cache.get(acquire_key, dsn_or_conn=self.handler.vars.static.cache_config):
            self.handler.cache.set(
                acquire_key,
                acquire_identifier,
                dsn_or_conn=self.handler.vars.static.cache_config,
                expire_in=60*3)
        else:
            options['proxy'] = True
            return self.render(*statics, **options)

        filename = '%s.%s' % (self.handler.helper.crypto.sha224_hash(self.handler.helper.random.uuid()), ext)
        use_systmp = False

        if use_systmp:
            output = tempfile.NamedTemporaryFile(mode='w', prefix=filename, suffix='.%s' % ext, delete=False)
            tempname = output.name
            output.close()
        else:
            output = open(os.path.join(self.handler.vars.static.combined_path, filename), 'w')
            tempname = output.name
            output.close()

        compressed = self.handler.vars.compressor.compress(self.handler.vars.static.path, statics, ext, tempname)

        # If failed compression.
        if not compressed:
            options['proxy'] = True
            return self.render(*statics, **options)

        # AWS not configured, compressed files are stored local storage.
        if not self.handler.vars.static.aws_configured:
            prepared = template % {
                'url': self.handler.vars.compressor.helper.url.urlparse.urljoin(
                    self.handler.vars.static.combined_prefix, filename)}

            test = os.path.exists(tempname)

        # AWS configured, upload compressed files to S3.
        else:
            content_length = os.path.getsize(tempname)
            filename = '%s/%s' % (self.handler.vars.static.combined_url, filename)

            if not self.handler.vars.static.aws_region:
                fp = open(tempname, 'r')

                s3bridge = self.handler.helper.aws.s3.connect(
                    self.handler.vars.static.aws_id, self.handler.vars.static.aws_secret)

                s3bridge.set_contents_from_file(self.handler.vars.static.aws_bucket, filename, fp)

                fp.close()

            else:
                self.handler.helper.aws.s3.set_contents_from_file(
                    aws_access_key_id=self.handler.vars.static.aws_id,
                    aws_secret_access_key=self.handler.vars.static.aws_secret,
                    bucket_name=self.handler.vars.static.aws_bucket,
                    region_name=self.handler.vars.static.aws_region,
                    key=filename,
                    fp=tempname, ExtraArgs={'ContentType': 'text/%s' % ('javascript' if ext == 'js' else ext)})

            os.remove(tempname)

            prepared_url = self.handler.vars.compressor.helper.url.urlparse.urljoin(
                self.handler.vars.static.aws_endpoint, filename)
            prepared = template % {'url': prepared_url}

            test = requests.get(prepared_url)
            test = False if test.status_code != 200 or (content_length and not len(test.text.strip())) else True

        if self.handler.cache.get(acquire_key, dsn_or_conn=self.handler.vars.static.cache_config) == acquire_identifier:
            if not test:
                options['proxy'] = True
                return self.render(*statics, **options)

            self.handler.vars.static.prepared.__setattr__(cache_key, prepared)
            self.handler.cache.set(cache_key, prepared, dsn_or_conn=self.handler.vars.static.cache_config)

        return prepared

    def _template(self, path, replaced=True):
        if replaced:
            url = self.handler.vars.compressor.helper.url.urlparse.urljoin(self.handler.vars.static.prefix, path)
            return self._template(path, replaced=False) % {'url': url}

        extension = path.split('?')[0].split('.')[-1].lower()

        if extension == 'css':
            return '<link rel="stylesheet" type="text/css" href="%(url)s" />'
        elif extension == 'js':
            return '<script type="text/javascript" src="%(url)s"></script>'
        else:
            raise NotImplementedError



# -*- coding: utf-8 -*-

"""

  dp for Tornado

  MVC Web Application Framework with Tornado
  http://github.com/why2pac/dp-tornado

  Copyright (c) 2015, why2pac <youngyongpark@gmail.com>

"""


import tornado.web
import tornado.ioloop
import tornado.httpserver
import tornado.options

import logging
import logging.handlers

import time
import os
import multiprocessing
import importlib

from dp_tornado.engine.bootstrap import Bootstrap as EngineBootstrap
from dp_tornado.engine.scheduler import Scheduler
from dp_tornado.engine.plugin.static import Compressor
from dp_tornado.engine.plugin.static import StaticURL
from dp_tornado.engine.plugin.prefix import PrefixURL
from dp_tornado.engine.plugin.pagination import Pagination
from dp_tornado.engine.plugin import ui_methods


class RestfulApplication(tornado.web.Application):
    def __init__(self, handlers, kwargs):
        self.startup_at = int(round(time.time() * 1000))

        kwargs['ui_modules']['Static'] = StaticURL
        kwargs['ui_modules']['Prefix'] = PrefixURL
        kwargs['ui_modules']['Pagination'] = Pagination
        kwargs['ui_methods'] = ui_methods

        super(RestfulApplication, self).__init__(handlers, **kwargs)


class Bootstrap(object):
    def run(self, **kwargs):
        # Initialize Logging
        logging.basicConfig(level=logging.DEBUG, format='[%(asctime)s][%(levelname)s] %(message)s')

        custom_scheduler = kwargs['scheduler'] if 'scheduler' in kwargs else None
        custom_service = kwargs['service'] if 'service' in kwargs else None
        custom_config_file = kwargs['config_file'] if 'config_file' in kwargs else None

        engine_path = os.path.join(os.path.dirname(os.path.realpath(__file__)))
        application_path = kwargs['application_path'] if 'application_path' in kwargs else None

        if 'initialize' in kwargs and kwargs['initialize']:
            EngineBootstrap.init_template(engine_path=engine_path, application_path=application_path)

        settings = EngineBootstrap.init_ini(application_path=application_path, ini_file=custom_config_file)

        logging.info('---------------------------------')
        logging.info('dp for Tornado has been started..')
        logging.info('---------------------------------')

        services_raw = [
            (r"/__scheduler__/(.*)", 'dp_tornado.engine.scheduler_handler.SchedulerHandler'),
            (r"/", None),
            (r"/(.*)", None),
        ]

        if custom_service:
            services_raw = custom_service + services_raw

        services = []
        default_handler = None

        for service in services_raw:
            if len(service) < 2:
                raise Exception('The specified service is invalid.')

            if service[1] is not None:
                s = str.split(service[1], '.')
                class_name = s.pop()
                module_path = '.'.join(s)

                handler_module = importlib.import_module(module_path)
                handler = getattr(handler_module, class_name)

            else:
                if default_handler is None:
                    handler_module = importlib.import_module('dp_tornado.engine.default_handler')
                    default_handler = getattr(handler_module, 'DefaultHandler')

                module_path = 'controller'
                handler = default_handler

            services.append((service[0], handler, dict(prefix=module_path)))

        # Clear combined files
        Compressor.clear(settings['combined_static_path'])
        num_processed = (tornado.options.options.num_processes
                         if tornado.options.options.num_processes else multiprocessing.cpu_count())

        logging.info('Server Mode : %s' % ('Production' if not tornado.options.options.debug else 'Debugging'))
        logging.info('Server time : %s' % time.strftime('%Y.%m.%d %H:%M:%S'))
        logging.info('Server Port : %s' % tornado.options.options.port)
        logging.info('Processors  : %s' % num_processed)
        logging.info('CPU Count   : %d' % multiprocessing.cpu_count())
        logging.info('---------------------------------')

        if custom_scheduler:
            scheduler = Scheduler(custom_scheduler)
            scheduler.start()

        else:
            scheduler = None

        application = RestfulApplication(services, settings)
        service = tornado.httpserver.HTTPServer(
            application,
            xheaders=True,
            max_body_size=tornado.options.options.max_body_size)
        service.bind(tornado.options.options.port, '')
        service.start(tornado.options.options.num_processes)

        import random
        application.identifier = random.randint(100000, 999999)

        try:
            instance = tornado.ioloop.IOLoop.instance()
            instance.__setattr__('startup_at', getattr(application, 'startup_at'))
            instance.start()

        except KeyboardInterrupt:
            if scheduler:
                scheduler.interrupted = True



# -*- coding: utf-8 -*-


from ..cache import CacheDriver as dpCacheDriver


import sys
import redis


__is_py3__ = True if sys.version_info[0] >= 3 else False


def decoded_value(val):
    if val and __is_py3__ and isinstance(val, bytes):
        return val.decode('utf8')

    return val


class RedisCacheDriver(dpCacheDriver):
    @staticmethod
    def getpool(config_dsn=None, host=None, port=None, database=None, user=None, password=None, maxconn=None):
        pool = redis.ConnectionPool(
            host=host,
            port=port if port is not None else 6379,
            db=database if database is not None else 0,
            max_connections=maxconn or None)

        return RedisCacheDriver(pool)

    def getconn(self):
        return RedisCacheDriver(self.pool, redis.Redis(connection_pool=self.pool))

    def flushall(self):
        return self.conn.flushall()

    def flushdb(self):
        return self.conn.flushdb()

    def get(self, key, expire_in, delete):
        if expire_in is None and not delete:
            return decoded_value(self.conn.get(key))

        else:
            p = self.conn.pipeline()
            p.get(key)

            if expire_in is not None:
                p.expire(key, int(expire_in))

            if delete:
                p.delete(key)

            ret = p.execute()

            if ret:
                return decoded_value(ret[0])
            
            else:
                return False

    def set(self, key, val, expire_in):
        if expire_in is None:
            return self.conn.set(key, val)

        else:
            p = self.conn.pipeline()
            p.set(key, val)
            p.expire(key, int(expire_in))
            return p.execute()

    def setnx(self, key, val, expire_in):
        setnx = self.conn.setnx(key, val)

        if not setnx:
            return False

        if expire_in is not None:
            expire = self.conn.expire(key, int(expire_in))

            if not expire:
                self.conn.delete(key)
                return False

        return True

    def delete(self, key):
        return self.conn.delete(key)

    def increase(self, key, amount, expire_in):
        if expire_in is None:
            return self.conn.incrby(key, amount)

        else:
            p = self.conn.pipeline()
            p.incrby(key, amount)
            p.expire(key, int(expire_in))
            return p.execute()

    def llen(self, key):
        return self.conn.llen(key)

    def lrange(self, key, start, stop):
        return self.conn.lrange(key, start, stop)

    def rpush(self, key, value, expire_in):
        if expire_in is None:
            return self.conn.rpush(key, value)

        else:
            p = self.conn.pipeline()
            p.rpush(key, value)
            p.expire(key, int(expire_in))
            return p.execute()

    def lpop(self, key):
        return self.conn.lpop(key)

    def blpop(self, key, timeout):
        blpop = self.conn.blpop(key, timeout)
        return blpop[1] if blpop else None

    def brpop(self, key, timeout):
        brpop = self.conn.brpop(key, timeout)
        return brpop[1] if brpop else None

    def lrem(self, key, count, value):
        lrem = self.conn.lrem(name=key, num=count, value=value)
        return lrem

    def lpush(self, key, value, expire_in):
        if expire_in is None:
            return self.conn.lpush(key, value)

        else:
            p = self.conn.pipeline()
            p.lpush(key, value)
            p.expire(key, int(expire_in))
            return p.execute()

    def rpop(self, key):
        return self.conn.rpop(key)

    def smembers(self, key):
        return self.conn.smembers(key)

    def scard(self, key, expire_in):
        if expire_in is None:
            return self.conn.scard(key)

        else:
            p = self.conn.pipeline()
            p.scard(key)
            p.expire(key, int(expire_in))
            ret = p.execute()

            if ret:
                return ret[0]
            else:
                return False

    def sadd(self, key, value, expire_in):
        if expire_in is None:
            return self.conn.sadd(key, value)

        else:
            p = self.conn.pipeline()
            p.sadd(key, value)
            p.expire(key, int(expire_in))
            return p.execute()

    def srem(self, key, value):
        if value:
            return self.conn.srem(key, value)
        else:
            return self.conn.spop(key)

    def publish(self, channel, message):
        return self.conn.publish(channel, message)

    def hlen(self, key, expire_in):
        if expire_in is None:
            return self.conn.hlen(key)

        else:
            p = self.conn.pipeline()
            p.hlen(key)
            p.expire(key, int(expire_in))
            ret = p.execute()

            if ret:
                return ret[0]
            else:
                return False

    def hgetall(self, key):
        return self.conn.hgetall(key)

    def hget(self, key, field):
        return self.conn.hget(key, field)

    def hset(self, key, field, val):
        return self.conn.hset(key, field, val)

    def hdel(self, key, field):
        return self.conn.hdel(key, field)

    def keys(self, pattern):
        return self.conn.keys(pattern)

    def dbsize(self):
        return self.conn.dbsize()

    def ttl(self, key):
        return self.conn.ttl(key)



# -*- coding: utf-8 -*-


from dp_tornado.engine.controller import Controller as dpController


class StarterController(dpController):
    def get(self):
        params = {
            'model': self.model.calc.number.add(10, 20),
            'helper': self.helper.calc.number.div(20, 10)
        }

        self.render('index.html', params)



# -*- coding: utf-8 -*-


import sys
import importlib

from dp_tornado.engine.bootstrap import Bootstrap as EngineBootstrap
from dp_tornado.engine.engine import Engine as dpEngine


if __name__ == '__main__':
    if len(sys.argv) <= 2:
        exit(0)

    args = EngineBootstrap.init_args()

    runner = None
    app_path = args.app_path
    module = args.scheduler_path
    timeout = args.scheduler_timeout or None

    sys.path.append(app_path)
    EngineBootstrap.init_ini(application_path=app_path, ini_file=None)

    try:
        module = importlib.import_module(module)

    except ImportError as e:
        print(e)
        print('The specified scheduler module is invalid. (%s)' % module)
        exit(1)

    try:
        runner = getattr(module, 'Scheduler')(timeout)

    except AttributeError:
        print('The specified scheduler module is not implemented. (%s)' % module)
        exit(1)

    try:
        runner.run()

    except Exception as e:
        dpEngine().logging.exception(e)

    dpEngine().logging.delegate_interrupt()



# -*- coding: utf-8 -*-


from .singleton import Singleton as dpSingleton
from .namer import Namer as dpNamer
from .loader import Loader as dpLoader


class Config(dpLoader, dpSingleton):
    conf = dpNamer()

    def __getattr__(self, name):
        return self.__getattr_inner__(name) or self.conf.__getattr__(name)
